@@ -549,6 +549,85 @@ query {
   }
 }
 
+By default ``__typename`` is only included when selecting :class:`Union`,
+if that should be included in every :class:`Type`, then you must specify
+``__typename__`` as a selected field. It's handled recursively:
+
+>>> op = Operation(Query)
+>>> op.repository(id='repo1').issues.__fields__('__typename__')
+>>> op
+query {
+  repository(id: "repo1") {
+    issues {
+      __typename
+      number
+      title
+      body
+      reporter {
+        __typename
+        login
+        name
+      }
+      assigned {
+        __typename
+        ... on User {
+          login
+          name
+        }
+        ... on Assignee {
+          email
+        }
+      }
+      commenters {
+        __typename
+        actors {
+          __typename
+          login
+        }
+      }
+    }
+  }
+}
+
+Or included using ``__typename__=True``:
+
+>>> op = Operation(Query)
+>>> op.repository(id='repo1').issues.__fields__(__typename__=True)
+>>> op
+query {
+  repository(id: "repo1") {
+    issues {
+      __typename
+      number
+      title
+      body
+      reporter {
+        __typename
+        login
+        name
+      }
+      assigned {
+        __typename
+        ... on User {
+          login
+          name
+        }
+        ... on Assignee {
+          email
+        }
+      }
+      commenters {
+        __typename
+        actors {
+          __typename
+          login
+        }
+      }
+    }
+  }
+}
+
+
 If a field of a container type (interface or type) is used without explicit
 fields as documented above, all of its fields will be added automatically.
 It will avoid dependency loops and limit the allowed nest depth to 2 by
@@ -650,6 +729,66 @@ query {
   }
 }
 
+If ``__typename`` is to be automatically selected, then use ``typename=True``:
+
+>>> print(op.__to_graphql__(auto_select_depth=4, typename=True))
+query {
+  repository(id: "repo1") {
+    __typename
+    id
+    name
+    owner {
+      __typename
+      login
+    }
+    issues {
+      __typename
+      number
+      title
+      body
+      reporter {
+        __typename
+        login
+        name
+      }
+      assigned {
+        __typename
+        ... on User {
+          login
+          name
+        }
+        ... on Assignee {
+          email
+        }
+      }
+      commenters {
+        __typename
+        actors {
+          __typename
+          login
+        }
+      }
+    }
+  }
+}
+
+
+.. note::
+
+  The built-in object type ``__typename`` would cause issues with Python's
+  name mangling as it would be translated to the private class member name.
+  In order to avoid this issue, whenever selecting GraphQL's ``__typename``
+  use the ``__typename__`` Python name. Example:
+
+  >>> op = Operation(Query)
+  >>> op.repository(id='repo1').__typename__()
+  __typename
+  >>> op
+  query {
+    repository(id: "repo1") {
+      __typename
+    }
+  }
 
 
 
@@ -780,6 +919,40 @@ repository(id: "repo1") {
 >>> obj.repository.name
 'Repo #1'
 
+And also if ``__typename__`` is selected:
+
+>>> op = Operation(Query)
+>>> op.repository(id='repo1', __typename__=True)
+repository(id: "repo1") {
+  __typename
+  id
+  name
+  owner {
+    __typename
+    login
+  }
+  issues {
+    __typename
+    number
+    title
+    body
+  }
+}
+>>> json_data = {'data': {
+...     'repository': {
+...         '__typename': 'Repository', 'id': 'repo1', 'name': 'Repo #1',
+...         'owner': {'__typename': 'Actor', 'login': 'name'},
+...         'issues': [{
+...             '__typename': 'Issue', 'number': 1, 'title': 'title',
+...         }],
+...     },
+... }}
+>>> obj = op + json_data
+>>> obj.repository
+Repository(__typename__='Repository', id='repo1', name='Repo #1', \
+owner=Actor(__typename__='Actor', login='name'), issues=[\
+Issue(__typename__='Issue', number=1, title='title')])
+
 
 Error Reporting
 ~~~~~~~~~~~~~~~
@@ -986,6 +1159,30 @@ it's going to behave as the interface type as well:
 >>> obj.repository.owner
 Actor(login='user')
 
+Auto-selection works on inline fragments (casts) as well:
+
+>>> op = Operation(Query)
+>>> repo = op.repository(id='repo1')
+>>> repo.owner.__as__(User).__fields__()
+>>> op
+query {
+  repository(id: "repo1") {
+    owner {
+      __typename
+      ... on User {
+        login
+        name
+      }
+    }
+  }
+}
+>>> json_data = {'data': {'repository': {'owner': {
+...    '__typename': 'User', 'login': 'user', 'name': 'User Name',
+... }}}}
+>>> obj = op + json_data
+>>> obj.repository.owner
+User(__typename__='User', login='user', name='User Name')
+
 
 Named Fragments
 ~~~~~~~~~~~~~~~
@@ -1132,6 +1329,33 @@ it's going to behave as the interface type as well:
 >>> obj.repository.owner
 Actor(login='user')
 
+Auto-selection works on fragments as well:
+
+
+>>> auto_sel_user = Fragment(User, 'AutoSelectedUser')
+>>> auto_sel_user.__fields__()
+>>> op = Operation(Query)
+>>> op.repository(id='repo1').owner.__fragment__(auto_sel_user)
+>>> op
+query {
+  repository(id: "repo1") {
+    owner {
+      __typename
+      ...AutoSelectedUser
+    }
+  }
+}
+fragment AutoSelectedUser on User {
+  login
+  name
+}
+>>> json_data = {'data': {'repository': {'owner': {
+...    '__typename': 'User', 'login': 'user', 'name': 'User Name',
+... }}}}
+>>> obj = op + json_data
+>>> obj.repository.owner
+User(__typename__='User', login='user', name='User Name')
+
 
 Utilities
 ~~~~~~~~~
@@ -1381,25 +1605,29 @@ class Selection:
 
     __slots__ = (
         '__alias__', '__field__', '__args__', '__field_selector',
-        '__selection_list',
+        '__selection_list', '__typename',
     )
 
-    def __init__(self, alias, field, args):
+    def __init__(self, alias, field, args, typename=None):
         self.__alias__ = alias
         self.__field__ = field
         self.__args__ = args
         self.__field_selector = {}
         self.__selection_list = None
+        self.__typename = typename
         if issubclass(field.type, BaseTypeWithTypename):
             self.__selection_list = SelectionList(field.type)
 
     def __get_selections_or_auto_select__(
-            self, auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH):
+            self, auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH,
+            typename=None):
         selections = self.__selection_list
         if selections is None or selections:
             return selections
+        if typename is None:
+            typename = self.__typename
         return self.__get_all_fields_selection_list(
-            auto_select_depth, [])
+            auto_select_depth, [], typename)
 
     def __len__(self):
         if self.__selection_list is not None:
@@ -1411,149 +1639,42 @@ class Selection:
             return iter(self.__selection_list)
         return iter((self,))
 
-    def __select_all_types(cls, union_type, depth, trail):
-        recursive = len(trail) < depth
+    def __get_all_fields_selection_list(self, depth, trail, typename):
+        '''Create a new SelectionList, select all fields and return it'''
+        selections = SelectionList(self.__field__.type)
+        selections.__select_all__(depth, trail, typename, typename)
+        return selections
 
-        q = SelectionList(union_type)
-        q.__typename__()
-        for t in union_type:
-            if recursive and t not in trail:
-                sel = q.__as__(t)
-                for x in cls.__select_all_fields(t, depth, trail):
-                    sel += x
-        return q
-
-    def __select_all_fields(cls, container_type, depth, trail):
-        recursive = len(trail) < depth
-
-        q = SelectionList(container_type)
-        for f in container_type:
-            sel = Selection(None, f, {})
-            if issubclass(f.type, BaseTypeWithTypename):
-                if recursive and f.type not in trail:
-                    # change the locally created selection list to the
-                    # auto-selected one. This must be explicit here so
-                    # it doesn't affect __to_graphql__(), that one must not
-                    # affect the actual selection it's operating on!
-                    sel.__selection_list = \
-                        sel.__get_all_fields_selection_list(depth, trail)
-                else:
-                    sel = None
-
-            if sel:
-                q += sel
-        return q
-
-    def __get_all_fields_selection_list(self, depth, trail):
-        t = self.__field__.type
-        trail.append(t)
-
-        if issubclass(t, Union):
-            q = self.__select_all_types(t, depth, trail)
-        else:
-            q = self.__select_all_fields(t, depth, trail)
-
-        trail.pop()
-        return q
+    def __select_all__(self, depth, trail, typename):
+        '''Select all fields in the current SelectionList'''
+        self.__selection_list.__select_all__(depth, trail, typename, typename)
 
     def __fields__(self, *names, **names_and_args):
         '''Select fields of a container type.
 
-        This is a helper to automate selection of fields of container
-        types, such as giving a list of names to include, with or
-        without parameters (passed as a mapping ``name=args``).
-
-        If no arguments are given, all fields are included.
-
-        If the keyword argument ``__exclude__`` is given a list of
-        names, then all but those fields will be
-        included. Alternatively one can exclude fields using
-        ``name=None`` or ``name=False`` as keyword argument.
-
-        If a list of names is given as positional arguments, then only
-        those names will be included. Alternatively one can include
-        fields using ``name=True``. To include fields with selection
-        parameters, then use ``name=dict(...)`` or ``name=list(...)``.
-        To include fields without arguments and with aliases, use the
-        shortcut ``name='alias'``.
-
-        .. code-block:: python
-
-          # just field1 and field2
-          parent.field.child.__fields__('field1', 'field2')
-          parent.field.child.__fields__(field1=True, field2=True)
-
-          # field1 with parameters
-          parent.field.child.__fields__(field1=dict(param1='value1'))
-
-          # field1 renamed (aliased) to alias1
-          parent.field.child.__fields__(field1='alias1')
-
-          # all but field2
-          parent.field.child.__fields__(field2=False)
-          parent.field.child.__fields__(field2=None)
-          parent.field.child.__fields__(__exclude__=('field2',))
+        See :func:`sgqlc.operation.SelectionList.__fields__`.
         '''
-        exclude = self.__fields_gen_excludes(names_and_args)
-
-        if not names and not names_and_args:
-            self.__fields_add_all(exclude)
-            return
-
-        self.__fields_add_names(names)
-        self.__fields_add_names_and_args(names_and_args)
-
-    def __fields_gen_excludes(self, names_and_args):
-        exclude = []
-        if '__exclude__' in names_and_args:
-            exclude = names_and_args.pop('__exclude__')
-
-        for k, v in tuple(names_and_args.items()):  # force copy
-            if v is False or v is None:
-                exclude.append(k)
-                del names_and_args[k]
-
-        return exclude
-
-    def __fields_add_all(self, exclude):
-        for f in self.__field__.type:
-            if f.name not in exclude:
-                self[f.name]()
-
-    def __fields_add_names(self, names):
-        for n in names:
-            self[n]()
-
-    def __fields_add_names_and_args(self, names_and_args):
-        for n, args in names_and_args.items():
-            if args and not isinstance(args, dict):
-                if isinstance(args, (tuple, list)):
-                    args = dict(args)
-                elif isinstance(args, str):
-                    args = {'__alias__': args}
-                else:
-                    args = {}
-            self[n](**args)
+        self.__selection_list.__fields__(*names, **names_and_args)
 
     def __to_graphql__(self, indent=0, indent_string='  ',
-                       auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH):
+                       auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH,
+                       typename=None):
         prefix = indent_string * indent
 
-        alias = ''
-        if self.__alias__:
-            alias = self.__alias__ + ': '
-
+        alias = self.__alias__ + ': ' if self.__alias__ else ''
         args = self.__field__.args.__to_graphql_input__(
             self.__args__, indent, indent_string)
 
         query = ''
         if self.__selection_list is not None:
+            if typename is None:
+                typename = self.__typename
             selections = self.__selection_list
             if not selections:
                 selections = self.__get_all_fields_selection_list(
-                    auto_select_depth, [])
+                    auto_select_depth, [], typename)
             query = ' ' + selections.__to_graphql__(
-                indent, indent_string, auto_select_depth)
+                indent, indent_string, auto_select_depth, typename)
         return prefix + alias + self.__field__.graphql_name + args + query
 
     def __collect_fragments__(self, fragments):
@@ -1681,10 +1802,8 @@ class Selector:
 
         To provide an alias, use ``__alias__`` keyword argument.
         '''
-        alias = None
-        if '__alias__' in args:
-            alias = args.pop('__alias__')
-
+        alias = args.pop('__alias__') if '__alias__' in args else None
+        typename = args.pop('__typename__') if '__typename__' in args else None
         s = self.__selections.get(alias)
         if s is not None:
             if not args:
@@ -1693,7 +1812,8 @@ class Selector:
                 ('%s already have a selection %s. '
                  'Maybe use __alias__ as param?') % (self.__field__, s))
 
-        s = self.__selections[alias] = Selection(alias, self.__field__, args)
+        s = self.__selections[alias] = Selection(
+            alias, self.__field__, args, typename)
         self.__parent__ += s
         return s
 
@@ -1841,20 +1961,21 @@ class SelectionList:
         return bytes(self.__to_graphql__(indent_string=''), 'utf-8')
 
     def __to_graphql__(self, indent=0, indent_string='  ',
-                       auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH):
+                       auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH,
+                       typename=None):
         prefix = indent_string * indent
         next_indent = indent + 1
 
         s = ['{']
         for v in self.__selections:
             s.append(v.__to_graphql__(
-                next_indent, indent_string, auto_select_depth,
+                next_indent, indent_string, auto_select_depth, typename,
             ))
 
         next_prefix = prefix + indent_string
         for v in self.__casts.values():
             s.append(next_prefix + v.__to_graphql__(
-                next_indent, indent_string, auto_select_depth,
+                next_indent, indent_string, auto_select_depth, typename,
             ))
         for lst in self.__fragments.values():
             for v in lst:
@@ -1933,7 +2054,7 @@ class SelectionList:
 
         sl = InlineFragmentSelectionList(typ)
         self.__casts[typ.__name__] = sl
-        self['__typename__']()
+        self.__typename__()
         return sl
 
     def __fragment__(self, fragment):
@@ -1941,14 +2062,158 @@ class SelectionList:
         self.__fragments.setdefault(
             fragment.__type__.__name__, []
         ).append(fragment)
-        self['__typename__']()
+        self.__typename__()
+
+    def __select_all_types(self, depth, trail, child_typename):
+        recursive = len(trail) < depth
+
+        self.__typename__()  # this one is required in unions
+
+        for t in self.__type:
+            if recursive and t not in trail:
+                sel = self.__as__(t)
+                sel.__select_all__(depth, trail, False, child_typename)
+
+    def __select_all_fields(self, depth, trail, self_typename, child_typename):
+        recursive = len(trail) < depth
+
+        if self_typename:
+            self.__typename__()
+
+        for f in self.__type:
+            if not issubclass(f.type, BaseTypeWithTypename):
+                self += Selection(None, f, {})
+            elif recursive and f.type not in trail:
+                sel = Selection(None, f, {})
+                sel.__select_all__(depth, trail, child_typename)
+                if sel:
+                    self += sel
+
+    def __select_all__(self, depth, trail, self_typename, child_typename):
+        trail.append(self.__type)
+
+        if issubclass(self.__type, Union):
+            self.__select_all_types(depth, trail, child_typename)
+        else:
+            self.__select_all_fields(depth, trail,
+                                     self_typename, child_typename)
+
+        trail.pop()
+
+    def __fields__(self, *names, **names_and_args):
+        '''Select fields of a container type.
+
+        This is a helper to automate selection of fields of container
+        types, such as giving a list of names to include, with or
+        without parameters (passed as a mapping ``name=args``).
+
+        If no arguments are given, all fields are included.
+
+        If the keyword argument ``__exclude__`` is given a list of
+        names, then all but those fields will be
+        included. Alternatively one can exclude fields using
+        ``name=None`` or ``name=False`` as keyword argument.
+
+        If a list of names is given as positional arguments, then only
+        those names will be included. Alternatively one can include
+        fields using ``name=True``. To include fields with selection
+        parameters, then use ``name=dict(...)`` or ``name=list(...)``.
+        To include fields without arguments and with aliases, use the
+        shortcut ``name='alias'``.
+
+        The special built-in field ``__typename`` is not selected by default.
+        In order to select it, provide ``__typename__=True`` as a parameter.
+
+        .. code-block:: python
+
+          # just field1 and field2
+          parent.field.child.__fields__('field1', 'field2')
+          parent.field.child.__fields__(field1=True, field2=True)
+
+          # field1 with parameters
+          parent.field.child.__fields__(field1=dict(param1='value1'))
+
+          # field1 renamed (aliased) to alias1
+          parent.field.child.__fields__(field1='alias1')
+
+          # all but field2
+          parent.field.child.__fields__(field2=False)
+          parent.field.child.__fields__(field2=None)
+          parent.field.child.__fields__(__exclude__=('field2',))
+
+          # all and also include __typename
+          parent.field.child.__fields__(__typename__=True)
+          parent.field.child.__fields__('__typename__')
+        '''
+        names = list(names)
+        exclude = self.__fields_gen_excludes(names_and_args)
+        typename = self.__fields_has_typename(names, names_and_args, exclude)
+        if typename:
+            self['__typename__']()
+
+        if not names and not names_and_args:
+            self.__fields_add_all(exclude, typename)
+            return
+
+        self.__fields_add_names(names, typename)
+        self.__fields_add_names_and_args(names_and_args, typename)
+
+    @staticmethod
+    def __fields_gen_excludes(names_and_args):
+        exclude = []
+        if '__exclude__' in names_and_args:
+            exclude = names_and_args.pop('__exclude__')
+
+        for k, v in tuple(names_and_args.items()):  # force copy
+            if v is False or v is None:
+                exclude.append(k)
+                del names_and_args[k]
+
+        return exclude
+
+    @staticmethod
+    def __fields_has_typename(names, names_and_args, exclude):
+        if '__typename__' in exclude:
+            return False  # pragma: no cover
+
+        try:
+            names.remove('__typename__')
+            return True
+        except ValueError:
+            pass
+
+        try:
+            return bool(names_and_args.pop('__typename__'))
+        except KeyError:
+            return False
+
+    def __fields_add_all(self, exclude, typename):
+        for f in self.__type:
+            if f.name not in exclude:
+                self[f.name](__typename__=typename)
+
+    def __fields_add_names(self, names, typename):
+        for n in names:
+            self[n](__typename__=typename)
+
+    def __fields_add_names_and_args(self, names_and_args, typename):
+        for n, args in names_and_args.items():
+            if args and not isinstance(args, dict):
+                if isinstance(args, (tuple, list)):
+                    args = dict(args)
+                elif isinstance(args, str):
+                    args = {'__alias__': args}
+                else:
+                    args = {}
+            self[n](__typename__=typename, **args)
 
 
 class InlineFragmentSelectionList(SelectionList):
     def __to_graphql__(self, indent=0, indent_string='  ',
-                       auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH):
+                       auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH,
+                       typename=None):
         selection = SelectionList.__to_graphql__(
-            self, indent, indent_string, auto_select_depth)
+            self, indent, indent_string, auto_select_depth, typename)
         return '... on %s %s' % (self.__type__, selection)
 
 
@@ -1967,9 +2232,10 @@ class Fragment(SelectionList):
         return '...%s' % (self.__name,)
 
     def __to_graphql__(self, indent=0, indent_string='  ',
-                       auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH):
+                       auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH,
+                       typename=None):
         selection = SelectionList.__to_graphql__(
-            self, indent, indent_string, auto_select_depth)
+            self, indent, indent_string, auto_select_depth, typename)
         return 'fragment %s on %s %s' % (
             self.__name, self.__type__, selection)
 
@@ -2190,26 +2456,24 @@ class Operation:
             )
 
     def __to_graphql__(self, indent=0, indent_string='  ',
-                       auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH):
+                       auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH,
+                       typename=None):
         prefix = indent_string * indent
         kind = self.__kind
-        name = ''
-        if self.__name:
-            name = ' ' + self.__name
-
+        name = ' ' + self.__name if self.__name else ''
         args = self.__args.__to_graphql__(indent, indent_string)
         selections = self.__selection_list.__to_graphql__(
-            indent, indent_string, auto_select_depth)
+            indent, indent_string, auto_select_depth, typename)
 
-        frags_gql = []
         fragments = self.__selection_list.__collect_fragments__()
-        for fragment in fragments.values():
-            frags_gql.append(fragment.__to_graphql__(indent, indent_string,
-                             auto_select_depth))
+        frags_gql = [
+            fragment.__to_graphql__(
+                indent, indent_string, auto_select_depth, typename
+            )
+            for fragment in fragments.values()
+        ]
 
-        frags = ''
-        if frags_gql:
-            frags = '\n' + '\n'.join(frags_gql)
+        frags = '\n' + '\n'.join(frags_gql) if frags_gql else ''
         return prefix + kind + name + args + ' ' + selections + frags
 
     def __iter__(self):
@@ -2238,10 +2502,7 @@ class Operation:
 
     def __add__(self, other):
         errors = other.get('errors')
-        ex = None
-        if errors:
-            ex = GraphQLErrors(errors)
-
+        ex = GraphQLErrors(errors) if errors else None
         data = other.get('data')
         if not data:
             if not ex:  # pragma: no cover
